<?php

namespace Drupal\entity_cache_rebuild\Controller;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Drupal\entity_cache_rebuild\Event\EntityCacheRebuildEvent;
use Symfony\Component\HttpFoundation\RedirectResponse;

class CacheRebuild extends ControllerBase {

  public function rebuild(RouteMatchInterface $route_match) {
    \Drupal::service('page_cache_kill_switch')->trigger();
    $route = $route_match->getRouteObject();

    $entity_type_id = $route->getOption('entity_type_id');
    $entity_id = $route_match->getParameter($entity_type_id);

    $entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->load($entity_id);
    $cache_tags = $entity->getCacheTags();
    $message = "Cache rebuilt for $entity_type_id entity $entity_id.";

    \Drupal::moduleHandler()->alter('entity_cache_rebuild', $cache_tags, $message, $entity);

    Cache::invalidateTags($cache_tags);
    \Drupal::messenger()->addMessage($message);

    $route_name = "entity.$entity_type_id.canonical";
    $route_parameters = [$entity_type_id => $entity_id];
    $url = Url::fromRoute($route_name, $route_parameters)->toString();
    return new RedirectResponse($url);
  }

}
